/******************************* main.c *******************************/
/*
ECE 495C, Spring 2008
Team 10

Prototype using 9S12C32 to control an HP Photosmart R707

*/            

#include <hidef.h> 
#include <mc9s12c32.h>
#pragma LINK_INFO DERIVATIVE "mc9s12c32"

#include "sci.h"
#include "pll.h"
#include "timer.h"
#include "sd.h"
#include "debug.h"
#include "camera.h"
#include "servo.h"
#include "radio.h"

/*
Microcontroller Connections:

Camera:
 PTT2  - On/Off
 PTT3  - Auto focus
 PTT4  - Shutter
 PTT5  - Video
 PTAD6 - Power-on sense

microSD:
 PTM5 - SCLK
 PTM4 - MOSI
 PTM3 - CS bar - controlled separately in code
 PTM2 - MISO   
 PTM1 - Card inserted when low
 
Servo:
 PTT0 - made useless
 PTT1 - 16bit PWM out

*/

unsigned char SD_Buffer[SD_BLOCK_SIZE];

void PrintTime(unsigned long t_s, unsigned long t_e) 
{
   t_e -= t_s;
   SCI_OutString("Time: ");
   SCI_OutULInt(t_e);
   SCI_OutString("\r\n");
}

void main(void) {
   unsigned char keyPress;	
   unsigned short i,j;	
   int prev=0, curr=0;
   unsigned char duty;
   short duty16;
   
   char *replace = "OMG, h4x0red!!!";
   char *pos;
   
   // for timing purposes
   unsigned long t[4];
   										   
   DDRT |= 0xFF;     // PortT bits 0-7 are output
  
   // use PTAD for digital input
   ATDDIEN = 0xFF;
   
     
   // Initialize all subsystems
   SCI_Init(19200); 
   //SCI_Init(115200);  
   PLL_Init();   
   Timer_Initialize(0,1); 
   Camera_Init();
   Servo_Init();
   Radio_Init();
/*   
   SD_Init();
   
   SCI_OutString("Initializing SD card...\r\n");
   t[0] = TimeStamp();
   i = SD_Card_Init();
   t[1] = TimeStamp();
   PrintTime(t[0], t[1]);
   
   PTT = 0;
   
   SCI_OutString("Reading block 645\r\n");
   t[0] = TimeStamp();
   i = SD_Read_Block(SD_Buffer, 645);   
   t[1] = TimeStamp();
   PrintTime(t[0], t[1]);
   
   SCI_OutString("Before: ");
   for (i=0; i<30; i++)
   {
      SCI_OutChar(SD_Buffer[i]);
   }
   SCI_OutString("\r\n");
   
   // modify the buffer
   pos = replace;
   i = 6;
   while (*replace) 
   {
      SD_Buffer[i] = *replace;
      replace++;
      i++;
   }  
   
   SCI_OutString("After:  ");
   for (i=0; i<30; i++)
   {
      SCI_OutChar(SD_Buffer[i]);
   }
   SCI_OutString("\r\n");
   
   //for (;;);
      
   
   SCI_OutString("Writing edited block 645\r\n");  
   t[0] = TimeStamp();  
   i = SD_Write_Block(SD_Buffer, 645);
   t[1] = TimeStamp();
   PrintTime(t[0], t[1]);
   
   
   for (j=645; j<1000; j++)
   {	
      t[0] = TimeStamp();
      i = SD_Read_Block(SD_Buffer, j);
      t[1] = TimeStamp();
      PrintTime(t[0], t[1]);
           		      
      SCI_OutString("Contents of memory block ");
      SCI_OutUInt(j);
      SCI_OutString("\r\n");
      
      // print out a hex dump
      for (i=1; i<=SD_BLOCK_SIZE; i++)
      {
         SCI_OutHex(SD_Buffer[i-1]);
         if ( (i%16) == 0 )
            SCI_OutString("\r\n");
         else if ( (i%8) == 0 )
            SCI_OutString("  ");
         else
            SCI_OutChar(' ');
      }
      SCI_OutString("\r\n");
      SCI_OutString("\r\n");
      
      // wait for button press
      for(;;)
      {         
         if (PTAD & 0x80) curr=0; else curr=1;
         if (curr & !prev) break;
         prev = curr;
      }
      
   }
   
  */ 
  
   /*
   // loop forever, testing 8bit PWM output
   for(;;)
   {
      // increase duty cycle
      for (duty=0; duty<255; duty++)
      {
         PWM_Duty8(0, duty);
         PWM_Duty8(1, 255-duty);  
         Timer_Wait1ms(4); 
      }
            
      // decrease duty cycle
      for (duty=255; duty>0; duty--)
      {
         PWM_Duty8(0, duty);
         PWM_Duty8(1, 255-duty);  
         Timer_Wait1ms(4); 
      }
   }
   */	  
   
     
   
  
   
   for(;;)
   {       
      // Test radio input
      
      
      if (Radio_ReadyFlag(7))		  
      {
         Radio_DisassertFlag(7);
         SCI_OutUInt(Radio_PulseWidth(7)>>3);
         SCI_OutString("\r\n");
        
         Timer_Wait1ms(50);
      }
   }
   
   
   
   
   duty16 = 4500;
   PWM_Duty16(0, duty16);
   
   for(;;)
   {    
   
      // Test servo output interface      
      
      	          
      keyPress = SCI_InChar();       
         
      if (keyPress == '1') duty16+=1;
      else if (keyPress == '2') duty16+=10;
      else if (keyPress == '3') duty16+=100;
      else if (keyPress == '4') duty16+=1000;
      else if (keyPress == '5') duty16=14720;
      else if (keyPress == 'q') duty16-=1;
      else if (keyPress == 'w') duty16-=10;
      else if (keyPress == 'e') duty16-=100;
      else if (keyPress == 'r') duty16-=1000;
      else if (keyPress == 't') duty16=3320;
      
      
      SCI_OutUInt(duty16);
      SCI_OutString("\r\n");       
      
          
   }
   
   
   // loop forever, testing 16bit PWM output
   for(;;)
   {
            
      // increase duty cycle
      for (duty16=3000; duty16<6000; duty16++)
      {
         PWM_Duty16(0, duty16);
         for (i=0; i<1000; i++);                         
      }
      
      Timer_Wait1ms(10);
      
      // increase duty cycle      
      for (duty16=6000; duty16<3000; duty16--)
      {
         PWM_Duty16(0, duty16);
         for (i=0; i<1000; i++);                         
      }	       
     
      Timer_Wait1ms(10);             
   }
   
   
   // loop forever, testing 16bit PWM output
   for(;;)
   {
            
      // increase duty cycle
      for (duty16=SERVO_IN_MIN; duty16<SERVO_IN_MAX; duty16++)
      {
         Servo_Duty(0, duty16);         
         Timer_Wait1ms(1); 
      }
      
      Timer_Wait1ms(1000);
      
      // increase duty cycle
      for (duty16=SERVO_IN_MAX; duty16>SERVO_IN_MIN; duty16--)
      {
         Servo_Duty(0, duty16);         
         Timer_Wait1ms(1); 
      }  
      
      Timer_Wait1ms(1000);             
   }
   
  
  
  
   // loop forever
   for(;;)
   {    
   
      // Test R707 Camera interface
      if (SCI_DATA_READY)
      {	          
         keyPress = SCI_InChar();
         keyPress &= ~0x20; // convert to uppercase
         
         if (keyPress == 'O') Camera_Send(CAMERA_COMMAND_ON);
         else if (keyPress == 'P') Camera_Send(CAMERA_COMMAND_OFF);
         else if (keyPress == 'A') Camera_Send(CAMERA_COMMAND_AF);
         else if (keyPress == 'S') Camera_Send(CAMERA_COMMAND_SHUTTER);
         else if (keyPress == 'V') Camera_Send(CAMERA_COMMAND_VIDEO_START);
         else if (keyPress == 'B') Camera_Send(CAMERA_COMMAND_VIDEO_STOP);
         else if (keyPress == 'Z') Camera_Send(CAMERA_COMMAND_AF_STROBE);
         else if (keyPress == 'X') Camera_Send(CAMERA_COMMAND_NULL_STROBE);
      }
    
      Camera_HandleEvents();
      
      
      // end main loop
   }
  
/*  
   // loop forever
   for(;;)
   {    
   
      // Test R707 Camera interface
      
      keyPress = SCI_InChar();
      keyPress &= ~0x20; // convert to uppercase
      
      if (keyPress == 'O') Camera_TurnOn();
      else if (keyPress == 'P') Camera_TurnOff();
      else if (keyPress == 'A') Camera_AutoFocus();
      else if (keyPress == 'S') Camera_Shutter();
      else if (keyPress == 'V') Camera_VideoStart();
      else if (keyPress == 'B') Camera_VideoStop();
    
      
      //if (SD_IS_CARD_INSERTED()) 
      //{
      //   PTT |= 0x01;
      //}
      //else
      //{
      //   PTT &= ~0x01;
      //}
           
      
      // end main loop
   }
*/
   
   // end main function
}


